import java.net.InetAddress;
import java.net.Socket;
import java.io.FileInputStream;
import java.io.File;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;

import alice.tuprolog.Number;
import alice.tuprolog.Prolog;
import alice.tuprolog.SolveInfo;
import alice.tuprolog.Struct;
import alice.tuprolog.Theory;
import alice.tuprolog.lib.JavaLibrary;

public class TemperatureAgent {
	Prolog engine;
	public TemperatureAgent() throws Exception {
		engine = new Prolog();
		engine.setTheory(new Theory(new FileInputStream(new File("temperature_agent.pl"))));
	}
	public void registerArtifact(String name, Object artifact) throws Exception {
		JavaLibrary javaLibrary = (JavaLibrary) engine.getLibrary("alice.tuprolog.lib.JavaLibrary");
		javaLibrary.register(new Struct(name), artifact);
	}
	public void start() {
		int port = 55555;
		Socket s = null;

		try {
			s = new Socket(InetAddress.getLocalHost(), port);
			System.out.println(InetAddress.getLocalHost());
		} catch (Exception e) {
			System.out.println(e.getMessage());
		}

		try {
			PrintWriter toThermostat = new PrintWriter(new OutputStreamWriter(s.getOutputStream()));

			int temperature = -1;
			while (temperature != 0) {
				engine.solve("change_temperature.");
				SolveInfo query = engine.solve("read_temperature(T).");
				temperature = ((Number) query.getVarValue("T")).intValue();
				toThermostat.println(temperature);
			    toThermostat.flush(); // flush the remaining contents of the buffer
			    Thread.currentThread().sleep(5000);
			}

			toThermostat.close();
			s.close();
		} catch (Exception e) {
			System.out.println(e.getMessage());
		}
	}
}
