import java.net.Socket;
import java.net.ServerSocket;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.File;
import java.io.InputStreamReader;

import alice.tuprolog.Number;
import alice.tuprolog.Prolog;
import alice.tuprolog.SolveInfo;
import alice.tuprolog.Struct;
import alice.tuprolog.Theory;
import alice.tuprolog.lib.JavaLibrary;

public class ThermostatAgent {
	Prolog engine;
	public ThermostatAgent() throws Exception {
		engine = new Prolog();
		engine.setTheory(new Theory(new FileInputStream(new File("thermostat_agent.pl"))));
	}
	public void registerArtifact(String name, Object artifact) throws Exception {
		JavaLibrary javaLibrary = (JavaLibrary) engine.getLibrary("alice.tuprolog.lib.JavaLibrary");
		javaLibrary.register(new Struct(name), artifact);
	}
	public void start() {
		int port = 55555;
		try {
			ServerSocket ss = new ServerSocket(port);
			System.out.println(ss.getInetAddress());
			for (;;) {
				Socket client = ss.accept();
				System.out.println("Connection accepted");
				ClientThread t = new ClientThread(client, engine);
				t.start();
			}
		} catch (Exception e) {
			System.out.println(e.getMessage());
		}
	}

	static class ClientThread extends Thread {
		Socket client;
		Prolog engine;
		ClientThread(Socket client, Prolog engine) {
			this.client = client;
			this.engine = engine;
			System.out.println("Thread created");
		}
		public void run() {
			try {
				BufferedReader in = new BufferedReader(new InputStreamReader(client.getInputStream()));
				String line;
				while ((line = in.readLine()) != null) {
					System.out.println("Message: <" + line + ">");
					if (line.length() == 0)
						break;
					int temperature = Integer.parseInt(line);
					engine.solve("check_temperature(" + temperature + ").");
				}
				in.close();
				client.close();
			} catch (Exception e) {
				System.out.println(e.getMessage());
			}
		}
	}

}
